@@ -1,4 +1,3 @@
-
 # Copyright 2013 Red Hat, Inc.
 #
 #    Licensed under the Apache License, Version 2.0 (the "License"); you may
@@ -20,12 +19,12 @@ methods which may be invoked remotely by clients over a given transport.
 To create an RPC server, you supply a transport, target and a list of
 endpoints.
 
-A transport can be obtained simply by calling the get_transport() method::
+A transport can be obtained simply by calling the get_rpc_transport() method::
 
-    transport = messaging.get_transport(conf)
+    transport = messaging.get_rpc_transport(conf)
 
 which will load the appropriate transport driver according to the user's
-messaging configuration. See get_transport() for more details.
+messaging configuration. See get_rpc_transport() for more details.
 
 The target supplied when creating an RPC server expresses the topic, server
 name and - optionally - the exchange to listen on. See Target for more details
@@ -47,12 +46,8 @@ Server will send the returned value back to the requesting client via the
 transport.
 
 The executor parameter controls how incoming messages will be received and
-dispatched. By default, the most simple executor is used - the blocking
-executor.  This executor processes inbound RPC requests on the server's thread,
-blocking it from processing additional requests until it finishes with the
-current request. This includes time spent sending the reply message to the
-transport if the method returns a result.  Refer to the Executor documentation
-for descriptions of the other types of executors.
+dispatched. Refer to the Executor documentation for descriptions of the types
+of executors.
 
 *Note:* If the "eventlet" executor is used, the threading and time library need
 to be monkeypatched.
@@ -77,6 +72,13 @@ complete after the Server has been stopped.
 
 A simple example of an RPC server with multiple endpoints might be::
 
+    # NOTE(changzhi): We are using eventlet executor and
+    # time.sleep(1), therefore, the server code needs to be
+    # monkey-patched.
+
+    import eventlet
+    eventlet.monkey_patch()
+
     from oslo_config import cfg
     import oslo_messaging
     import time
@@ -98,14 +100,14 @@ A simple example of an RPC server with multiple endpoints might be::
         def test(self, ctx, arg):
             return arg
 
-    transport = oslo_messaging.get_transport(cfg.CONF)
+    transport = oslo_messaging.get_rpc_transport(cfg.CONF)
     target = oslo_messaging.Target(topic='test', server='server1')
     endpoints = [
         ServerControlEndpoint(None),
         TestEndpoint(),
     ]
     server = oslo_messaging.get_rpc_server(transport, target, endpoints,
-                                           executor='blocking')
+                                           executor='eventlet')
     try:
         server.start()
         while True:
@@ -118,27 +120,30 @@ A simple example of an RPC server with multiple endpoints might be::
 
 """
 
+import logging
+import sys
+
+from oslo_messaging import exceptions
+from oslo_messaging.rpc import dispatcher as rpc_dispatcher
+from oslo_messaging import server as msg_server
+from oslo_messaging import transport as msg_transport
+
 __all__ = [
     'get_rpc_server',
     'expected_exceptions',
     'expose'
 ]
 
-import logging
-import sys
-
-from debtcollector.updating import updated_kwarg_default_value
-
-from oslo_messaging._i18n import _LE
-from oslo_messaging.rpc import dispatcher as rpc_dispatcher
-from oslo_messaging import server as msg_server
-
 LOG = logging.getLogger(__name__)
 
 
 class RPCServer(msg_server.MessageHandlingServer):
     def __init__(self, transport, target, dispatcher, executor='blocking'):
         super(RPCServer, self).__init__(transport, dispatcher, executor)
+        if not isinstance(transport, msg_transport.RPCTransport):
+            LOG.warning("Using notification transport for RPC. Please use "
+                        "get_rpc_transport to obtain an RPC transport "
+                        "instance.")
         self._target = target
 
     def _create_listener(self):
@@ -146,10 +151,13 @@ class RPCServer(msg_server.MessageHandlingServer):
 
     def _process_incoming(self, incoming):
         message = incoming[0]
+
+        # TODO(sileht): We should remove that at some point and do
+        # this directly in the driver
         try:
             message.acknowledge()
         except Exception:
-            LOG.exception(_LE("Can not acknowledge message. Skip processing"))
+            LOG.exception("Can not acknowledge message. Skip processing")
             return
 
         failure = None
@@ -163,31 +171,28 @@ class RPCServer(msg_server.MessageHandlingServer):
             # by another exception raised by a log handler during
             # LOG.exception(). So keep a copy and delete it later.
             failure = sys.exc_info()
-            LOG.exception(_LE('Exception during message handling'))
+            LOG.exception('Exception during message handling')
 
         try:
             if failure is None:
                 message.reply(res)
             else:
                 message.reply(failure=failure)
+        except exceptions.MessageUndeliverable as e:
+            LOG.exception(
+                "MessageUndeliverable error, "
+                "source exception: %s, routing_key: %s, exchange: %s: ",
+                e.exception, e.routing_key, e.exchange
+            )
         except Exception:
-            LOG.exception(_LE("Can not send reply for message"))
+            LOG.exception("Can not send reply for message")
         finally:
-                # NOTE(dhellmann): Remove circular object reference
-                # between the current stack frame and the traceback in
-                # exc_info.
-                del failure
+            # NOTE(dhellmann): Remove circular object reference
+            # between the current stack frame and the traceback in
+            # exc_info.
+            del failure
 
 
-@updated_kwarg_default_value('access_policy', None,
-                             rpc_dispatcher.DefaultRPCAccessPolicy,
-                             message='access_policy defaults to '
-                                     'LegacyRPCAccessPolicy which '
-                                     'exposes private methods. Explicitly '
-                                     'set access_policy to '
-                                     'DefaultRPCAccessPolicy or '
-                                     'ExplicitRPCAccessPolicy.',
-                             version='?')
 def get_rpc_server(transport, target, endpoints,
                    executor='blocking', serializer=None, access_policy=None):
     """Construct an RPC server.
@@ -199,12 +204,12 @@ def get_rpc_server(transport, target, endpoints,
     :param endpoints: a list of endpoint objects
     :type endpoints: list
     :param executor: name of message executor - available values are
-                     'eventlet', 'blocking' and 'threading'
+                     'eventlet' and 'threading'
     :type executor: str
     :param serializer: an optional entity serializer
     :type serializer: Serializer
     :param access_policy: an optional access policy.
-           Defaults to LegacyRPCAccessPolicy
+           Defaults to DefaultRPCAccessPolicy
     :type access_policy: RPCAccessPolicyBase
     """
     dispatcher = rpc_dispatcher.RPCDispatcher(endpoints, serializer,
@@ -223,6 +228,7 @@ def expected_exceptions(*exceptions):
     ExpectedException, which is used internally by the RPC sever. The RPC
     client will see the original exception type.
     """
+
     def outer(func):
         def inner(*args, **kwargs):
             try:
@@ -235,7 +241,9 @@ def expected_exceptions(*exceptions):
             # ignored and thrown as normal.
             except exceptions:
                 raise rpc_dispatcher.ExpectedException()
+
         return inner
+
     return outer
 
 
