@@ -12,6 +12,7 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
+import copy
 import logging
 import os
 import select
@@ -50,10 +51,11 @@ if pyngus:
 # The Cyrus-based SASL tests can only be run if the installed version of proton
 # has been built with Cyrus SASL support.
 _proton = importutils.try_import("proton")
-CYRUS_ENABLED = (pyngus and pyngus.VERSION >= (2, 0, 0) and _proton
-                 and getattr(_proton.SASL, "extended", lambda: False)())
+CYRUS_ENABLED = (pyngus and pyngus.VERSION >= (2, 0, 0) and _proton and
+                 getattr(_proton.SASL, "extended", lambda: False)())
 # same with SSL
-SSL_ENABLED = (_proton and getattr(_proton.SSL, "present", lambda: False)())
+# SSL_ENABLED = (_proton and getattr(_proton.SSL, "present", lambda: False)())
+SSL_ENABLED = False
 
 LOG = logging.getLogger(__name__)
 
@@ -74,18 +76,18 @@ class _ListenerThread(threading.Thread):
         self.messages = moves.queue.Queue()
         self.daemon = True
         self.started = threading.Event()
-        self._done = False
+        self._done = threading.Event()
         self.start()
         self.started.wait()
 
     def run(self):
         LOG.debug("Listener started")
         self.started.set()
-        while not self._done:
+        while not self._done.is_set():
             for in_msg in self.listener.poll(timeout=0.5):
                 self.messages.put(in_msg)
                 self.msg_count -= 1
-                self._done = self.msg_count == 0
+                self.msg_count == 0 and self._done.set()
                 if self._msg_ack:
                     in_msg.acknowledge()
                     if in_msg.message.get('method') == 'echo':
@@ -108,16 +110,65 @@ class _ListenerThread(threading.Thread):
         return msgs
 
     def kill(self, timeout=30):
-        self._done = True
+        self._done.set()
         self.join(timeout)
 
 
+class _SlowResponder(_ListenerThread):
+    # an RPC listener that pauses delay seconds before replying
+    def __init__(self, listener, delay, msg_count=1):
+        self._delay = delay
+        super(_SlowResponder, self).__init__(listener, msg_count)
+
+    def run(self):
+        LOG.debug("_SlowResponder started")
+        self.started.set()
+        while not self._done.is_set():
+            for in_msg in self.listener.poll(timeout=0.5):
+                time.sleep(self._delay)
+                in_msg.acknowledge()
+                in_msg.reply(reply={'correlation-id':
+                                    in_msg.message.get('id')})
+                self.messages.put(in_msg)
+                self.msg_count -= 1
+                self.msg_count == 0 and self._done.set()
+
+
+class _CallMonitor(_ListenerThread):
+    # an RPC listener that generates heartbeats before
+    # replying.
+    def __init__(self, listener, delay, hb_count, msg_count=1):
+        self._delay = delay
+        self._hb_count = hb_count
+        super(_CallMonitor, self).__init__(listener, msg_count)
+
+    def run(self):
+        LOG.debug("_CallMonitor started")
+        self.started.set()
+        while not self._done.is_set():
+            for in_msg in self.listener.poll(timeout=0.5):
+                hb_rate = in_msg.client_timeout / 2.0
+                deadline = time.time() + self._delay
+                while deadline > time.time():
+                    if self._done.wait(hb_rate):
+                        return
+                    if self._hb_count > 0:
+                        in_msg.heartbeat()
+                        self._hb_count -= 1
+                in_msg.acknowledge()
+                in_msg.reply(reply={'correlation-id':
+                                    in_msg.message.get('id')})
+                self.messages.put(in_msg)
+                self.msg_count -= 1
+                self.msg_count == 0 and self._done.set()
+
+
 @testtools.skipUnless(pyngus, "proton modules not present")
 class TestProtonDriverLoad(test_utils.BaseTestCase):
 
     def setUp(self):
         super(TestProtonDriverLoad, self).setUp()
-        self.messaging_conf.transport_driver = 'amqp'
+        self.messaging_conf.transport_url = 'amqp://'
 
     def test_driver_load(self):
         transport = oslo_messaging.get_transport(self.conf)
@@ -363,27 +414,11 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
 
     def test_call_late_reply(self):
         """What happens if reply arrives after timeout?"""
-
-        class _SlowResponder(_ListenerThread):
-            def __init__(self, listener, delay):
-                self._delay = delay
-                super(_SlowResponder, self).__init__(listener, 1)
-
-            def run(self):
-                self.started.set()
-                while not self._done:
-                    for in_msg in self.listener.poll(timeout=0.5):
-                        time.sleep(self._delay)
-                        in_msg.acknowledge()
-                        in_msg.reply(reply={'correlation-id':
-                                            in_msg.message.get('id')})
-                        self.messages.put(in_msg)
-                        self._done = True
-
         driver = amqp_driver.ProtonDriver(self.conf, self._broker_url)
         target = oslo_messaging.Target(topic="test-topic")
         listener = _SlowResponder(
-            driver.listen(target, None, None)._poll_style_listener, 3)
+            driver.listen(target, None, None)._poll_style_listener,
+            delay=3)
 
         self.assertRaises(oslo_messaging.MessagingTimeout,
                           driver.send, target,
@@ -408,14 +443,14 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
 
             def run(self):
                 self.started.set()
-                while not self._done:
+                while not self._done.is_set():
                     for in_msg in self.listener.poll(timeout=0.5):
                         try:
                             raise RuntimeError("Oopsie!")
                         except RuntimeError:
                             in_msg.reply(reply=None,
                                          failure=sys.exc_info())
-                        self._done = True
+                        self._done.set()
 
         driver = amqp_driver.ProtonDriver(self.conf, self._broker_url)
         target = oslo_messaging.Target(topic="test-topic")
@@ -440,13 +475,13 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
 
             def run(self):
                 self.started.set()
-                while not self._done:
+                while not self._done.is_set():
                     for in_msg in self.listener.poll(timeout=0.5):
                         # reply will never be acked (simulate drop):
                         in_msg._reply_to = "!no-ack!"
                         in_msg.reply(reply={'correlation-id':
                                             in_msg.message.get("id")})
-                        self._done = True
+                        self._done.set()
 
         driver = amqp_driver.ProtonDriver(self.conf, self._broker_url)
         driver._default_reply_timeout = 1
@@ -553,6 +588,66 @@ class TestAmqpSend(_AmqpBrokerTestCaseAuto):
 
         driver.cleanup()
 
+    def test_call_monitor_ok(self):
+        # verify keepalive by delaying the reply > heartbeat interval
+        driver = amqp_driver.ProtonDriver(self.conf, self._broker_url)
+        target = oslo_messaging.Target(topic="test-topic")
+        listener = _CallMonitor(
+            driver.listen(target, None, None)._poll_style_listener,
+            delay=11,
+            hb_count=100)
+        rc = driver.send(target,
+                         {"context": True},
+                         {"method": "echo", "id": "1"},
+                         wait_for_reply=True,
+                         timeout=60,
+                         call_monitor_timeout=5)
+        self.assertIsNotNone(rc)
+        self.assertEqual("1", rc.get('correlation-id'))
+        listener.join(timeout=30)
+        self.assertFalse(listener.isAlive())
+        driver.cleanup()
+
+    def test_call_monitor_bad_no_heartbeat(self):
+        # verify call fails if keepalives stop coming
+        driver = amqp_driver.ProtonDriver(self.conf, self._broker_url)
+        target = oslo_messaging.Target(topic="test-topic")
+        listener = _CallMonitor(
+            driver.listen(target, None, None)._poll_style_listener,
+            delay=11,
+            hb_count=1)
+        self.assertRaises(oslo_messaging.MessagingTimeout,
+                          driver.send,
+                          target,
+                          {"context": True},
+                          {"method": "echo", "id": "1"},
+                          wait_for_reply=True,
+                          timeout=60,
+                          call_monitor_timeout=5)
+        listener.kill()
+        self.assertFalse(listener.isAlive())
+        driver.cleanup()
+
+    def test_call_monitor_bad_call_timeout(self):
+        # verify call fails if deadline hit regardless of heartbeat activity
+        driver = amqp_driver.ProtonDriver(self.conf, self._broker_url)
+        target = oslo_messaging.Target(topic="test-topic")
+        listener = _CallMonitor(
+            driver.listen(target, None, None)._poll_style_listener,
+            delay=20,
+            hb_count=100)
+        self.assertRaises(oslo_messaging.MessagingTimeout,
+                          driver.send,
+                          target,
+                          {"context": True},
+                          {"method": "echo", "id": "1"},
+                          wait_for_reply=True,
+                          timeout=11,
+                          call_monitor_timeout=5)
+        listener.kill()
+        self.assertFalse(listener.isAlive())
+        driver.cleanup()
+
 
 class TestAmqpNotification(_AmqpBrokerTestCaseAuto):
     """Test sending and receiving notifications."""
@@ -748,7 +843,7 @@ mech_list: ${mechs}
                                   sasl_config_dir=_dir,
                                   sasl_config_name="openstack")
         self._broker.start()
-        self.messaging_conf.transport_driver = 'amqp'
+        self.messaging_conf.transport_url = 'amqp://'
         self.conf = self.messaging_conf.conf
 
     def tearDown(self):
@@ -809,17 +904,6 @@ mech_list: ${mechs}
                           addr,
                           retry=0)
 
-    def test_authentication_default_username(self):
-        """Verify that a configured username/password is used if none appears
-        in the URL.
-        Deprecated: username password deprecated in favor of transport_url
-        """
-        addr = "amqp://%s:%d" % (self._broker.host, self._broker.port)
-        self.config(username="joe@myrealm",
-                    password="secret",
-                    group="oslo_messaging_amqp")
-        self._authentication_test(addr)
-
     def test_authentication_default_realm(self):
         """Verify that default realm is used if none present in username"""
         addr = "amqp://joe:secret@%s:%d" % (self._broker.host,
@@ -1221,10 +1305,10 @@ class TestAddressing(test_utils.BaseTestCase):
         s1_payload = [m.message.get('msg') for m in rl[0].get_messages()]
         s2_payload = [m.message.get('msg') for m in rl[1].get_messages()]
 
-        self.assertTrue("Server1" in s1_payload
-                        and "Server2" not in s1_payload)
-        self.assertTrue("Server2" in s2_payload
-                        and "Server1" not in s2_payload)
+        self.assertTrue("Server1" in s1_payload and
+                        "Server2" not in s1_payload)
+        self.assertTrue("Server2" in s2_payload and
+                        "Server1" not in s2_payload)
         self.assertEqual(s1_payload.count("Fanout"), 1)
         self.assertEqual(s2_payload.count("Fanout"), 1)
         self.assertEqual((s1_payload + s2_payload).count("Anycast1"), 1)
@@ -1419,16 +1503,24 @@ class TestMessageRetransmit(_AmqpBrokerTestCase):
                                                  None,
                                                  None)._poll_style_listener,
                                    1)
-        rc = driver.send(target, {"context": "whatever"},
-                         {"method": "echo", "id": "blah"},
-                         wait_for_reply=True,
-                         retry=2)  # initial send + up to 2 resends
-        self.assertIsNotNone(rc)
-        self.assertEqual(0, self._nack_count)
-        self.assertEqual(rc.get('correlation-id'), 'blah')
-        listener.join(timeout=30)
-        self.assertFalse(listener.isAlive())
-        driver.cleanup()
+        try:
+            rc = driver.send(target, {"context": "whatever"},
+                             {"method": "echo", "id": "blah"},
+                             wait_for_reply=True,
+                             retry=2)  # initial send + up to 2 resends
+        except Exception:
+            # Some test runs are expected to raise an exception,
+            # clean up the listener since no message was received
+            listener.kill(timeout=30)
+            raise
+        else:
+            self.assertIsNotNone(rc)
+            self.assertEqual(0, self._nack_count)
+            self.assertEqual(rc.get('correlation-id'), 'blah')
+            listener.join(timeout=30)
+        finally:
+            self.assertFalse(listener.isAlive())
+            driver.cleanup()
 
     def test_released(self):
         # should retry and succeed
@@ -1471,11 +1563,10 @@ class TestSSL(test_utils.BaseTestCase):
         conf['c_name'] = 'client.com'
         self._ssl_config = conf
         ssl_setup = [
-            # create self-signed CA key and certificate:
-            Template('openssl genrsa -out ${ca_key} 2048').substitute(conf),
-            Template('openssl req -x509 -key ${ca_key} -subj'
-                     ' "/CN=Trusted.CA.com" -out'
-                     ' ${ca_cert}').substitute(conf),
+            # create self-signed CA certificate:
+            Template('openssl req -x509 -nodes -newkey rsa:2048'
+                     ' -subj "/CN=Trusted.CA.com" -keyout ${ca_key}'
+                     ' -out ${ca_cert}').substitute(conf),
             # create Server key and certificate:
             Template('openssl genrsa -out ${s_key} 2048').substitute(conf),
             Template('openssl req -new -key ${s_key} -subj /CN=${s_name}'
@@ -1507,19 +1598,12 @@ class TestSSL(test_utils.BaseTestCase):
                 self._tmpdir = None
                 self.skipTest("OpenSSL tools not installed - skipping")
 
-    def test_server_ok(self):
-        # test client authenticates server
-        self._broker = FakeBroker(self.conf.oslo_messaging_amqp,
-                                  sock_addr=self._ssl_config['s_name'],
-                                  ssl_config=self._ssl_config)
-        url = oslo_messaging.TransportURL.parse(self.conf, "amqp://%s:%d" %
-                                                (self._broker.host,
-                                                 self._broker.port))
+    def _ssl_server_ok(self, url):
         self._broker.start()
-
         self.config(ssl_ca_file=self._ssl_config['ca_cert'],
                     group='oslo_messaging_amqp')
-        driver = amqp_driver.ProtonDriver(self.conf, url)
+        tport_url = oslo_messaging.TransportURL.parse(self.conf, url)
+        driver = amqp_driver.ProtonDriver(self.conf, tport_url)
         target = oslo_messaging.Target(topic="test-topic")
         listener = _ListenerThread(
             driver.listen(target, None, None)._poll_style_listener, 1)
@@ -1533,6 +1617,34 @@ class TestSSL(test_utils.BaseTestCase):
         self.assertFalse(listener.isAlive())
         driver.cleanup()
 
+    def test_server_ok(self):
+        # test client authenticates server
+        self._broker = FakeBroker(self.conf.oslo_messaging_amqp,
+                                  sock_addr=self._ssl_config['s_name'],
+                                  ssl_config=self._ssl_config)
+        url = "amqp://%s:%d" % (self._broker.host, self._broker.port)
+        self._ssl_server_ok(url)
+
+    def test_server_ignore_vhost_ok(self):
+        # test client authenticates server and ignores vhost
+        self._broker = FakeBroker(self.conf.oslo_messaging_amqp,
+                                  sock_addr=self._ssl_config['s_name'],
+                                  ssl_config=self._ssl_config)
+        url = "amqp://%s:%d/my-vhost" % (self._broker.host, self._broker.port)
+        self._ssl_server_ok(url)
+
+    def test_server_check_vhost_ok(self):
+        # test client authenticates server using vhost as CN
+        # Use 'Invalid' from bad_cert CN
+        self.config(ssl_verify_vhost=True, group='oslo_messaging_amqp')
+        self._ssl_config['s_cert'] = self._ssl_config['bad_cert']
+        self._ssl_config['s_key'] = self._ssl_config['bad_key']
+        self._broker = FakeBroker(self.conf.oslo_messaging_amqp,
+                                  sock_addr=self._ssl_config['s_name'],
+                                  ssl_config=self._ssl_config)
+        url = "amqp://%s:%d/Invalid" % (self._broker.host, self._broker.port)
+        self._ssl_server_ok(url)
+
     @mock.patch('ssl.get_default_verify_paths')
     def test_server_ok_with_ssl_set_in_transport_url(self, mock_verify_paths):
         # test client authenticates server
@@ -1622,6 +1734,90 @@ class TestSSL(test_utils.BaseTestCase):
         super(TestSSL, self).tearDown()
 
 
+@testtools.skipUnless(pyngus, "proton modules not present")
+class TestVHost(_AmqpBrokerTestCaseAuto):
+    """Verify the pseudo virtual host behavior"""
+
+    def _vhost_test(self):
+        """Verify that all messaging for a particular vhost stays on that vhost
+        """
+        self.config(pseudo_vhost=True,
+                    group="oslo_messaging_amqp")
+
+        vhosts = ["None", "HOSTA", "HOSTB", "HOSTC"]
+        target = oslo_messaging.Target(topic="test-topic")
+        fanout = oslo_messaging.Target(topic="test-topic", fanout=True)
+
+        listeners = {}
+        ldrivers = {}
+        sdrivers = {}
+
+        replies = {}
+        msgs = {}
+
+        for vhost in vhosts:
+            url = copy.copy(self._broker_url)
+            url.virtual_host = vhost if vhost != "None" else None
+            ldriver = amqp_driver.ProtonDriver(self.conf, url)
+            listeners[vhost] = _ListenerThread(
+                ldriver.listen(target, None, None)._poll_style_listener,
+                10)
+            ldrivers[vhost] = ldriver
+            sdrivers[vhost] = amqp_driver.ProtonDriver(self.conf, url)
+            replies[vhost] = []
+            msgs[vhost] = []
+
+        # send a fanout and a single rpc call to each listener
+        for vhost in vhosts:
+            if vhost == "HOSTC":   # expect no messages to HOSTC
+                continue
+            sdrivers[vhost].send(fanout,
+                                 {"context": vhost},
+                                 {"vhost": vhost,
+                                  "fanout": True,
+                                  "id": vhost})
+            replies[vhost].append(sdrivers[vhost].send(target,
+                                                       {"context": vhost},
+                                                       {"method": "echo",
+                                                        "id": vhost},
+                                                       wait_for_reply=True))
+        time.sleep(1)
+
+        for vhost in vhosts:
+            msgs[vhost] += listeners[vhost].get_messages()
+            if vhost == "HOSTC":
+                # HOSTC should get nothing
+                self.assertEqual(0, len(msgs[vhost]))
+                self.assertEqual(0, len(replies[vhost]))
+                continue
+
+            self.assertEqual(2, len(msgs[vhost]))
+            for m in msgs[vhost]:
+                # the id must match the vhost
+                self.assertEqual(vhost, m.message.get("id"))
+            self.assertEqual(1, len(replies[vhost]))
+            for m in replies[vhost]:
+                # same for correlation id
+                self.assertEqual(vhost, m.get("correlation-id"))
+
+        for vhost in vhosts:
+            listeners[vhost].kill()
+            ldrivers[vhost].cleanup
+            sdrivers[vhost].cleanup()
+
+    def test_vhost_routing(self):
+        """Test vhost using routable addresses
+        """
+        self.config(addressing_mode='routable', group="oslo_messaging_amqp")
+        self._vhost_test()
+
+    def test_vhost_legacy(self):
+        """Test vhost using legacy addresses
+        """
+        self.config(addressing_mode='legacy', group="oslo_messaging_amqp")
+        self._vhost_test()
+
+
 class FakeBroker(threading.Thread):
     """A test AMQP message 'broker'."""
 
